package com.persagy.server.controllers.objects

import com.persagy.server.synchronize.MessageHandler
import com.persagy.server.datacenter.models.entities.assistant.ShaftFuncType
import com.persagy.server.datacenter.models.entities.assistant.ShaftWithCount
import com.persagy.server.datacenter.models.entities.objects.Shaft
import com.persagy.server.datacenter.models.entities.objects.ZoneSpace
import com.persagy.server.datacenter.models.entities.rel.RelationSpaceShaft
import com.persagy.server.datacenter.models.entities.task.BuildingDic
import com.persagy.server.services.objects.ShaftService
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.service.SPageContext
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*

/**
 * 竖井接口
 *
 * @author 张维新
 */
@Api(tags = ["008、竖井"])
@RestController
@RequestMapping("/object/shaft")
class ShaftController {
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(ShaftController::class.java)
    } // Companion object

    /** 获取消息对象实例 */
    @Autowired
    val messageHandler: MessageHandler? = null

    /**
     * 创建竖井数据
     *
     * @param   request     竖井对象
     * @return              创建结果信息
     */
    @ApiOperation(value = "创建竖井数据", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody request: SCreateRequest<Shaft>): SCreateResponse<Shaft> {
        logger.debug("ShaftCreate =${request.toJson()}")
        return ShaftService.createList(request)
    } // Function Create()

    /**
     * 根据id删除竖井
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     */
    @ApiOperation(value = "根据id删除项目", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<Shaft>): SBaseResponse {
        return ShaftService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 更新竖井信息
     *
     * @param   request     更新的内容对象
     * @return              更新的结果
     */
    @ApiOperation(value = "更新竖井信息", notes = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<Shaft>): SBaseResponse {
        logger.debug("ShaftUpdate =${request.toJson()}")
        return ShaftService.updateList(request)
    } // Function update()

    /**
     * 查询竖井信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
    @ApiOperation(value = "查询竖井信息", notes = "级联查询：竖井内的设备（equipmentList）、竖井内的系统（systemList）、竖井联通关系（shaftThroughList）、竖井内的业务空间（spaceList）")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<Shaft> {
        logger.debug("ShaftQuery =${request.toJson()}")
        return ShaftService.pageQuery(request, arrayListOf(SFilter.eq("projectId", SPageContext.getHeader("ProjectId").toString())))
    } // Function query()

    /**
     * 查询竖井信息-包含统计信息
     *
     * @param       request
     * @return      竖井统计信息
     */
    @ApiOperation(value = "查询竖井信息-包含统计信息", notes = "此方法可能会弃用")
    @PostMapping(value = ["/query-with-count"])
    fun queryWithCount(@RequestBody request: SQueryRequest): SQueryResponse<ShaftWithCount> {
        return ShaftService.queryWithCount(request)
    } // Function queryWithCount()

    /**
     * 关联元空间
     *
     * @param       request   条件
     * @return                关联反馈
     */
    @ApiOperation(value = "关联元空间", notes = "可能移至关系")
    @PostMapping(value = ["/relate-space"])
    fun relateSpace(@RequestBody request: RelationSpaceShaft): SBaseResponse {
        return ShaftService.relateSpace(request, messageHandler)
    } // Function relateSpace()

    /**
     * 查询竖井功能
     *
     * @return      竖井功能列表
     */
    @ApiOperation(value = "查询项目下已有胡竖井类型", notes = "此方法可能会弃用")
    @PostMapping(value = ["/query-shaft-functype"])
    fun queryShaftFuncType(): SQueryResponse<ShaftFuncType> {
        return ShaftService.queryShaftFuncType()
    } // Function queryShaftFuncType()

//    @ApiOperation(value="查询没有和当前竖井贯通的其它竖井", notes="")
//    @PostMapping(value = ["/unshaft-through-shaft"])
//    fun  unShaftThroughShaft(shaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<Shaft> {
//        return ShaftService.unShaftThroughShaft(shaftId,request)
//    } // Function unSpaceEq()

    /**
     * 查询和当前竖井有关联的业务空间
     *
     * @param   ShaftId        竖井id
     * @param   request        查询条件
     */
    @ApiOperation(value="查询和当前竖井有关联的业务空间", notes="shaftId 为竖井id")
    @PostMapping(value = ["/shaft-zoneSpaceBaseTable"])
    fun shaftZoneSpaceBaseTable(ShaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<ZoneSpace>{
        return ShaftService.shaftZoneSpaceBaseTable(ShaftId, request)
    } // Fun shaftZoneSpaceBaseTable()

    /**
     * 查询和当前竖井没有关联的业务空间
     *
     * @param   ShaftId        竖井id
     * @param   request        查询条件
     */
    @ApiOperation(value="查询和当前竖井没有关联的业务空间", notes="shaftId 为竖井id")
    @PostMapping(value = ["/un-shaft-zoneSpaceBaseTable"])
    fun unShaftZoneSpaceBaseTable(ShaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<ZoneSpace>{
        return ShaftService.unShaftZoneSpaceBaseTable(ShaftId, request)
    } // Fun shaftZoneSpaceBaseTable()


    /**
     * 查询和当前竖井有关联的业务空间所在建筑及楼层信息
     *
     * @param   ShaftId        竖井id
     * @param   ObjectType     空间类型
     */
    @ApiOperation(value="查询和当前竖井有关联的业务空间所在建筑及楼层信息", notes="shaftId 为竖井id")
    @GetMapping(value = ["/shaft-zoneSpaceBaseTable-building"])
    fun shaftZoneSpaceBaseTableBuilding(ShaftId: String,ObjectType:String?): SQueryResponse<BuildingDic>{
        return ShaftService.shaftZoneSpaceBaseTableBuilding(ShaftId,ObjectType)
    } // Fun shaftZoneSpaceBaseTable()


//    /**
//     * 查询没有和当前竖井关联的设备
//     *
//     * @param   ShaftId        竖井id
//     * @param   request        查询条件
//     */
//    @ApiOperation(value="查询没有和当前竖井关联的设备", notes="ShaftId 为竖井id")
//    @PostMapping(value = ["/un-shaft-equip"])
//    fun unShaftEquip( ShaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<Equipment>{
//        return ShaftService.unShaftEquip(ShaftId, request)
//    } // Fun unShaftEquip()
//
//    /**
//     * 查询没有和当前竖井关联的其他竖井
//     *
//     * @param    ShaftId        竖井id
//     * @param   request        查询条件
//     */
//    @ApiOperation(value="查询没有和当前竖井关联的其他竖井", notes="shaftId 竖井id")
//    @PostMapping(value = ["/un-shaft-othershaft"])
//    fun unShaftOtherShaft( ShaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<Shaft>{
//        return ShaftService.unShaftOtherShaft(ShaftId, request)
//    } //  Fun unShaftOtherShaft()
//
//    /**
//     * 查询没有和当前竖井关联的系统
//     *
//     * @param   ShaftId        竖井id
//     * @param   request        查询条件
//     */
//    @ApiOperation(value="查询没有和当前竖井关联的系统", notes="ShaftId 为竖井id")
//    @PostMapping(value = ["/un-shaft-sys"])
//    fun unShaftSys( ShaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<GeneralSystem>{
//        return ShaftService.unShaftSys(ShaftId, request)
//    } // Fun unShaftSys()


    /**
     * 根据条件查询统计数量
     */
    @ApiOperation(value = "根据条件查询统计数量", notes = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("ProjectId").toString()))
        return ShaftService.count(request,withFilterList)
    } // Function count()





} // Class ShaftController